﻿/*
**  Image-LP class
**  version1.9.1
*/
/*
	Copyright(c)2014-2019 株式会社トラスト・ソフトウェア・システム,All Rights Reserved.
*/

using System;
using System.Text;
using System.Runtime.InteropServices;

namespace PDFTools.PdfImagerLP
{
	public delegate void DelegateRendering(int pageNum);
	public delegate void DelegateRendered(int pageNum);
	public delegate void DelegateProgress(int lines, int totalLines);
	public delegate int DelegateDrawStrokePath(IntPtr spa, IntPtr pixels);
	public delegate void DelegateDefineImageTile(IntPtr ita, IntPtr pixels);
	public delegate void DelegateDefineMaskImageTile(IntPtr ita, IntPtr pixels);
	public delegate int DelegateDrawTiledImage(IntPtr ia1, IntPtr pixels1, IntPtr ia2, IntPtr pixels2);

	public enum PictureOpt
	{
		PICTURE_RGB = 0,		/*RBGカラー指定*/
		PICTURE_GRAY,			/*グレースケール指定*/
		PICTURE_DITHER,			/*ディザ変換指定*/
		PICTURE_BW,				/*２値色指定*/
		PICTURE_FAXG3,			/*CCITT FAX G3圧縮指定*/
		PICTURE_FAXG4,			/*CCITT FAX G4圧縮指定*/

		COMPRESS_NONE,			/*TIFF変換の場合に圧縮なし*/
		COMPRESS_JPEG,			/*TIFF変換の場合にJPEG圧縮*/
		COMPRESS_DEFLATE,		/*TIFF変換の場合にDeflate圧縮*/
		COMPRESS_LZW,			/*TIFF変換の場合にLZW圧縮*/

		JPEG_QUALITY,			/*JPEG圧縮の品質*/

		RESOLUTION_DPI,			/*XDPI,YDPI双方同値の解像度*/
		RESOLUTION_XDPI,		/*XDPI 解像度*/
		RESOLUTION_YDPI,		/*YDPI 解像度*/
		CANVAS_XY,				/*キャンバスサイズ XY双方を指定*/
		CANVAS_X,				/*キャンバスサイズ X-方向のみを指定*/
		CANVAS_Y,				/*キャンバスサイズ Y-方向のみを指定*/
		PICTURE_XSIZE,			/*論理サイズ幅を指定*/
		PICTURE_YSIZE,			/*論理サイズ高さを指定*/
		PICTURE_SIZE_UNIT,		/*論理サイズの単位*/
		PICTURE_USE_MEDIABOX,	/*ページ境界をMediaBoxにする*/
		PICTURE_USE_CROPBOX,	/*ページ境界をCropBoxにする*/
		PICTURE_USE_BLEEDBOX,	/*ページ境界をBleedBoxにする*/
		PICTURE_USE_TRIMBOX,	/*ページ境界をTrimBoxにする*/
		PICTURE_USE_ARTBOX,		/*ページ境界をArtBoxにする*/
		PIXEL_SIZE_X,			/*ピクセル単位の画像幅*/
		PIXEL_SIZE_Y,			/*ピクセル単位の画像高さ*/
		FILL_PATH_AA,			/*フィルパスのアンチアイリアスを制御*/
		NOT_DEF_GLYPH = 28,		/*未定義グリフの処理 0:notdefを表示, 1:空白(gid=0)を表示, 2:この文字を無視*/
		CID_0_GLYPH = 28,   	/*NOT_DEF_GLYPHのエイリアス*/
		TRUNCATE_PAGE_BBOX,		/*ページ境界を整数化する際に小数部を切り捨て処理する(MediaBoxの場合を除く)*/
		PAGE_CLIP_VALUE,		/*ページをその領域より指定値ピクセル小さな領域でクリップする*/
		STROKEPATH_MINIMUM_LINE_WIDTH,	/*パスのストロークにおける最小線幅指定*/

		THRESHOLD_BITONAL,		/*32 二値化の閾値*/
		THRESHOLD_DITHER,		/*33 ディザ化の閾値*/

		PICTURE_CMYK = 36,		/*CMYK色空間*/
		ANTIALIAS_LEVEL,		/*アンチアイリアスのレベル*/
		DATA_COMPRESSION_LEVEL, /*データ圧縮のレベル*/
		PICTURE_USE_MEDIA_CROP = 43,	/*ページの境界をMediaBoxまたはCropBoxの小さい方にする*/
		OCG_USAGE = 44,			/*0:"view", 1:"Print", 2:"Export"*/
		TEXT_ANTIALIAS,			/*1:テキストのアンチアイリアス有効(既定)、0:無効*/
		XY_PIXEL_SIZE_MEM = 47,	/*出来上がり画像データの物理サイズ (縦横でそれぞれのDPIに設定される)*/
		REPLACE_TR2_MODE = 48,	/*テキストのFill the Stroke描画を変更 0:従来どおり、1:Fill描画、2:Stroke描画*/
		WATERMARK_TEXT_FILL = 51, /*テキストウォータマークをFillで作成*/
		WATERMARK_TEXT_STROKE,	/*テキストウォータマークをStrokeで作成*/
		BITMAP_IMG_FORMAT = 53,	/*BMP形式のデータフォーマット 0:Windows、1:OS/2*/
		CANVAS_ORIGIN = 54,		/*キャンバスサイズ変更の原点指定*/
		USE_ZERO_LINE_WIDTH=55,	/*幅ゼロのストロークに対し、0:幅１ピクセルに変更(旧来互換)、1:アンチアイリアスのレベルに変更*/
		ROTATE_PAGE_IMAGE=60,	/*PDF文書の現在ページを一時的に回転させる*/

		LOGICAL_CPU_COUNT = 101,	/*論理的なCPUの総数*/
		THREAD_SPLIT_PAGE = 102,	/*ページを複数のスレッドで実行*/
		THREAD_SPLIT_PAGE_COUNT,	/*ページを分割処理するスレッドの数*/
		THREAD_EACH_PAGE,			/*各ページを個別のスレッドで実行*/
		THREAD_EACH_PAGE_COUNT,		/*格ページを処理するスレッドの数*/

		STROKEPATH_ENABLE_CALLBACK = 1001,	/*ストロークパスのコールバックを有効にする*/
		TILE_IMAGE_ENABLE_CALLBACK,			/*タイル画像のコールバックを有効にする*/
		TILE_MASK_IMG_ENABLE_CALLBACK,		/*タイルマスク画像のコールバックを有効にする*/
		TILING_ENABLE_CALLBACK,				/*タイリングのコールバックを有効にする*/
		TILING_IGNORE_PROCESS,				/*タイリング処理を完全に無視する(この機能は非サポート)*/
		SUPPRESS_ALPHA_CHANNEL_ON_CB,		/*コールバック時の画像データにアルファチャネル値を含めない*/
		ALLOW_PDF_14_TO_17 = 1101,	/*PDF文書バージョン 1.4->1.7*/
		ALLOW_PDF_10_TO_17,			/*PDF文書バージョン 1.0->1.7*/
		ALLOW_PDF_14_TO_20,			/*PDF文書バージョン 1.4->1.7,2.0*/
		ALLOW_PDF_ALL,				/*PDF文書バージョン 1.0->1.7,2.0*/
	}

	public enum DocumentOpt
	{
		PERMISSION_FLAG		= 1201,			/*文書のパーミッションフラグ 未設定の場合は-1が戻る*/
		OCG_COUNT			= 1202,			/*文書のOCG数を戻す*/
		GLYPH_NOT_FOUND_COUNT = 1203,		/*グリフが検索できなかった数*/
		PDF_AUTHOR			= 1204,
		NEEDS_PASSWORD		= 1205,			/*0:文書は非暗号化、1:既定のパスワードで復号、2:復号にパスワードが必要*/
		PRINT_RESTRICTION	= 1207,			/*制限された印刷解像度フラグ値 1:限定された(低解像度)、0:無制限(高解像度)*/
		DOCUMENT_TITLE      = 1211,			/*PDF文書のタイトル*/
		CREATION_DATE,						/*生成日時*/
		MODIFICATION_DATE,					/*最新の変更日時*/
		ENCRYPT_VERSION,					/*文書の暗号化バージョン*/
		ENCRYPT_REVISION,					/*文書の暗号化レビジョン*/
		PDF_VERSION,						/*PDF文書のバージョン*/

		//no longer support
		MLP_PERMISSION_FLAG	= 1201,			/*文書のパーミッションフラグ 未設定の場合は-1が戻る*/
		MLP_OCG_COUNT		= 1202,			/*文書のOCG数を戻す*/
		MLP_GLYPH_NOT_FOUND_COUNT = 1203,	/*グリフが検索できなかった数*/
		MLP_PDF_AUTHOR		= 1204,
		MLP_NEEDS_PASSWORD	= 1205,			/*0:文書は非暗号化、1:既定のパスワードで復号、2:復号にパスワードが必要*/
		MLP_OCG_PROPERTY	= 1206,			/*OCGプロパティ 非サポート*/
		MLP_PRINT_RESTRICTION = 1207,	/*制限された印刷解像度フラグ値 1:限定された(低解像度)、0:無制限(高解像度)*/
		MLP_DOCUMENT_TITLE	= 1211,	/*PDF文書のタイトル*/
		MLP_CREATION_DATE,			/*生成日時*/
		MLP_MODIFICATION_DATE,		/*最新の変更日時*/
		MLP_ENCRYPT_VERSION,		/*文書の暗号化バージョン*/
		MLP_ENCRYPT_REVISION,		/*文書の暗号化レビジョン*/
		MLP_PDF_VERSION,			/*PDF文書のバージョン*/
	}

	public enum LibraryOpt
	{
		LOGICAL_CPU_COUNT = 101,			/*論理的なCPUの総数 _未使用_ */
		THREAD_SPLIT_PAGE = 102,			/*ページを複数のスレッドで実行*/
		THREAD_SPLIT_PAGE_COUNT,			/*ページを分割処理するスレッドの数*/
		THREAD_EACH_PAGE,					/*各ページを個別のスレッドで実行*/
		THREAD_EACH_PAGE_COUNT,				/*各ページを処理するスレッドの数*/

		LICENSE_ERROR_INFO = 1201,			/*不正なライセンスの詳細情報*/

		//no longer support
		MLP_LICENSE_ERROR_INFO = 1201,		/*不正なライセンスの詳細情報*/
	}

	public enum PictureUnit
	{
		PICTURE_UNIT_PERCENT	= 1,		/*画像の論理サイズを指定する際の単位がパーセント*/
		PICTURE_UNIT_INCH		= 2,		/*画像の論理サイズを指定する際の単位がインチ*/
		PICTURE_UNIT_MM			= 3,		/*画像の論理サイズを指定する際の単位がミリメートル[mm]*/

		//no longer support
		MLP_PICTURE_UNIT_PERCENT	= 1,		/*画像の論理サイズを指定する際の単位がパーセント*/
		MLP_PICTURE_UNIT_INCH		= 2,		/*画像の論理サイズを指定する際の単位がインチ*/
		MLP_PICTURE_UNIT_MM			= 3,		/*画像の論理サイズを指定する際の単位がミリメートル[mm]*/
	}

	public enum CallbackMode
	{
		RENDERING_CB			= 1,		/*ページの解析終了 / レンダリングの開始を知らせるコールバック*/
		PROGRESS_CB				= 2,		/*指定した行数のレンダリング（画像格納）が終了したことを知らせるコールバック*/
		RENDERED_CB				= 3,		/*レンダリング終了を知らせるコールバック*/

		DRAW_STROKE_PATH_CB		= 101,		/*パスのストローキングが発生したときのコールバック*/
		DEFINE_IMAGE_TILE_CB	= 102,		/*タイルが定義されたときのコールバック*/
		DEFINE_MASK_IMAGE_TILE_CB= 103,		/*タイルのマスクが定義されたときのコールバック*/
		DRAW_TILED_IMAGE_CB		= 104,		/*タイリングされた画像を貼り付るときのコールバック*/

		//no longer support
		MLP_RENDERING_CB			= 1,		/*ページの解析終了 / レンダリングの開始を知らせるコールバック*/
		MLP_PROGRESS_CB				= 2,		/*指定した行数のレンダリング（画像格納）が終了したことを知らせるコールバック*/
		MLP_RENDERED_CB				= 3,		/*レンダリング終了を知らせるコールバック*/

		MLP_DRAW_STROKE_PATH_CB		= 101,		/*パスのストローキングが発生したときのコールバック*/
		MLP_DEFINE_IMAGE_TILE_CB	= 102,		/*タイルが定義されたときのコールバック*/
		MLP_DEFINE_MASK_IMAGE_TILE_CB= 103,		/*タイルのマスクが定義されたときのコールバック*/
		MLP_DRAW_TILED_IMAGE_CB		= 104,		/*タイリングされた画像を貼り付るときのコールバック*/
	}

	public enum error
	{
		MLP_NO_ERROR				=	0,
		MLP_ALREADY_INITIALIZED		=  -1,		/*既に初期化されています*/
		MLP_NOT_INITIALIZED			=  -2,		/*初期化できませんでした。または、初期化してません*/
		MLP_INIT_FILE_OPEN_ERROR	=  -3,		/*初期化ファイルを読めません*/
		MLP_INIT_DATA_LOAD_ERROR	=  -3,		/*初期化データをローディングできません*/
		MLP_LICENSE_ERROR			=  -4,		/*不正なライセンスです*/
		MLP_ALREADY_OPENED			=  -5,		/*既にオープンしています*/
		MLP_FILE_OPEN_ERROR			=  -6,		/*オープンできません*/
		MLP_FILE_IS_NOT_PDF			=  -7,		/*PDFではないようです*/
		MLP_FILE_NOT_DECRYPTED		=  -8,		/*復号できません*/
		MLP_FILE_NOT_OPENED			=  -9,		/*オープンしていません*/
		MLP_OPEN_ERROR				= -10,		/*オープンでエラー*/
		MLP_PDF_PARSE_ERROR			= -11,		/*PDFの解析エラー*/
		MLP_PDF_HAS_NOT_PAGE		= -12,		/*PDFにページがありません*/
		MLP_INVALID_PAGE_NUMBER		= -13,		/*無効なページ番号 または、無効なページ指定*/
		MLP_INVALID_RESOLUTION		= -14,		/*無効な解像度*/
		MLP_INVALID_QUALITY			= -15,		/*無効なJPEG品質*/
		MLP_NO_OUTPUT_FILE			= -16,		/*出力ファイルがありません*/
		MLP_TOO_LARGE_PIXEL			= -17,		/*ピクセル マップが大きすぎます*/
		MLP_DRAW_ERROR				= -18,		/*描画領域確保に失敗。または、画像の書き出しに失敗しました*/
		MLP_MEMORY_ERROR			= -20,		/*メモリー領域を確保できません*/
		MLP_INVALID_CANVAS_SIZE		= -21,		/*無効なキャンバスサイズ*/
		MLP_INVALID_ARG_VALUE		= -22,		/*無効な引数の値*/
		MLP_INVALID_PICTURE_TYPE	= -23,		/*無効な画像形式*/
		MLP_INVALID_CMD				= -24,		/*無効なコマンド*/
		MLP_NO_DATA					= -25,		/*データがありません*/
		MLP_FAIL_TO_GET_PAGE		= -27,		/*ページの取得に失敗しました*/
		MLP_INVALID_DATA			= -30,		/*データが不正*/
		MLP_NO_LICENSE_KEY			= -31,		/*ライセンスキーが指定されていません*/
		MLP_UNUSABLE_LICENSE		= -32,		/*このバージョンでは使えないライセンスです*/
		MLP_EXPIRED_LICENSE			= -33,		/*失効したライセンスです*/
		MLP_ILLEGAL_OS_LICENSE		= -34,		/*このOSでは使えないライセンスです*/
		MLP_ILLIGUL_OS_LICENSE		= -34,		/*このOSでは使えないライセンスです*/
		MLP_OUT_OF_BOUND			= -35,		/*値が範囲外です*/
		MLP_COLOR_PROFILE_NOT_FOUND	= -36,		/*カラープロファイルを読み込めません*/
		MLP_INVALID_VER_COLOR_PROFILE=-37,		/*ver.2ではないカラープロファイル*/
		MLP_INVALID_COLOR_PROFILE	= -38,		/*不正なカラープロファイル*/
		MLP_FAIL_TO_COUNT_PAGES		= -39,		/*ページ数の取得に失敗しました*/
		MLP_INVALID_PDF_VERSION		= -40,		/*不正なPDF文書のバージョンです*/
		MLP_FONT_NOT_FOUND			= -41,		/*フォントが見つかりません*/
		MLP_ALT_FONT_NOT_FOUND		= -42,		/*代替フォントが見つかりません*/
		MLP_FONT_FILE_NOT_OPENED	= -43,		/*フォントファイルを開けません*/
		MLP_FONT_NOT_LOADED			= -44,		/*フォントをロードできません*/
		MLP_GLYPH_NOT_FOUND			= -51,		/*グリフを検索できません*/
	}

	public enum TiffCompressMode
	{
		TIFF_COMPRESS_NONE = 0,	/*TIFF変換で圧縮なし*/
		TIFF_COMPRESS_JPEG,		/*TIFF変換でJPEG圧縮*/
		TIFF_COMPRESS_DEFLATE,	/*TIFF変換でDeflate圧縮*/
		TIFF_COMPRESS_LZW,		/*TIFF変換でLZW圧縮*/

		//no longer support
		MLP_TIFF_COMPRESS_NONE = 0,	/*TIFF変換で圧縮なし*/
		MLP_TIFF_COMPRESS_JPEG,		/*TIFF変換でJPEG圧縮*/
		MLP_TIFF_COMPRESS_DEFLATE,	/*TIFF変換でDeflate圧縮*/
		MLP_TIFF_COMPRESS_LZW,		/*TIFF変換でLZW圧縮*/
	}

	public enum Cid0GlyphMode
	{
		SHOW_NOTDEF_GLYPH		= 0,	/*.notdefグリフを表示*/
		REPLACE_TO_BLANK_GLYPH	= 1,	/*Spaceグリフに変換*/
		IGNORE_CID_0_GLYPH		= 2,	/*CID=0を無視する*/
	}

	public enum BoundaryBox
	{
		MEDIA_BOX	= 0,
		CROP_BOX	= 1,
		BLEED_BOX	= 2,
		TRIM_BOX	= 3,
		ART_BOX		= 4,
	}

	public	enum WatermarkLayer {
		WML_NO_PAINT	= 0,		/*非表示*/
		WML_FOREGROUND	= 1,		/*最前面*/
		WML_BACKGROUND	= 2,		/*最背面*/
	}

	public enum CanvasOrigin {		/*キャンバスサイズ変更の原点*/
		CORIGIN_CENTER		= 0,	/*センター*/
		CORIGIN_LEFT_TOP	= 1,	/*左上*/
		CORIGIN_LEFT_BOTTOM	= 2,	/*左下*/
		CORIGIN_RIGHT_TOP	= 3,	/*右上*/
		CORIGIN_RIGHT_BOTTOM= 4,	/*右下*/
	}

	public class MlpConstants
	{
		public const int MIN_RESOLUTION		= 50;
		public const int MAX_RESOLUTION		= 10000;
		public const int DEFAULT_RESOLUTION	= 150;
		public const int MIN_QUALITY		= 10;
		public const int MAX_QUALITY		= 90;
		public const int MIN_PICT_BUF_SIZE	= 50;
		public const int MAX_PICT_BUF_SIZE	= 1024;
		public const int MIN_PIXEL_SIZE		= 200;
		public const int MAX_PIXEL_SIZE		= 5000000;
		public const int DEFAULT_MULTILINE_TEXT_WIDGET_FONT_SIZE = 55;	/*マルチラインのTextWidget(Form Annotation)のフォントの既定サイズ(値は実サイズの１０倍)*/
		public const int CONVERT_CMYK_TO_RGB= 1;			/*CMYKからRGBへの変換*/
		public const int CONVERT_MOST_FAST	= 1;			/*最も速い*/
		public const int CONVERT_FAST		= 2;			/*速い*/
		public const int CONVERT_BY_COLOR_PROFILE = 3;		/*カラープロファイルによる変換*/
	}

	public class err
	{
		public const int MLP_NO_ERROR				=	0;
		public const int MLP_ALREADY_INITIALIZED	=  -1;		/*既に初期化されています*/
		public const int MLP_NOT_INITIALIZED		=  -2;		/*初期化できませんでした。または、初期化してません*/
		public const int MLP_INIT_FILE_OPEN_ERROR	=  -3;		/*初期化ファイルを読めません*/
		public const int MLP_INIT_DATA_LOAD_ERROR	=  -3;		/*初期化データをローディングできません*/
		public const int MLP_LICENSE_ERROR			=  -4;		/*不正なライセンスです*/
		public const int MLP_ALREADY_OPENED			=  -5;		/*既にオープンしています*/
		public const int MLP_FILE_OPEN_ERROR		=  -6;		/*オープンできません*/
		public const int MLP_FILE_IS_NOT_PDF		=  -7;		/*PDFではないようです*/
		public const int MLP_FILE_NOT_DECRYPTED		=  -8;		/*復号できません*/
		public const int MLP_FILE_NOT_OPENED		=  -9;		/*オープンしていません*/
		public const int MLP_OPEN_ERROR				= -10;		/*オープンでエラー*/
		public const int MLP_PDF_PARSE_ERROR		= -11;		/*PDFの解析エラー*/
		public const int MLP_PDF_HAS_NOT_PAGE		= -12;		/*PDFにページがありません*/
		public const int MLP_INVALID_PAGE_NUMBER	= -13;		/*無効なページ番号 または、無効なページ指定*/
		public const int MLP_INVALID_RESOLUTION		= -14;		/*無効な解像度*/
		public const int MLP_INVALID_QUALITY		= -15;		/*無効なJPEG品質*/
		public const int MLP_NO_OUTPUT_FILE			= -16;		/*出力ファイルがありません*/
		public const int MLP_TOO_LARGE_PIXEL		= -17;		/*ピクセル マップが大きすぎます*/
		public const int MLP_DRAW_ERROR				= -18;		/*描画領域確保に失敗。または、画像の書き出しに失敗しました*/
		public const int MLP_MEMORY_ERROR			= -20;		/*メモリー領域を確保できません*/
		public const int MLP_INVALID_ARG_VALUE      = -22;		/*無効な引数の値*/
		public const int MLP_INVALID_PICTURE_TYPE	= -23;		/*無効な画像形式*/
		public const int MLP_INVALID_CMD			= -24;		/*無効なコマンド*/
		public const int MLP_NO_DATA				= -25;		/*データがありません*/
		public const int MLP_FAIL_TO_GET_PAGE		= -27;		/*ページの取得に失敗しました*/
		public const int MLP_INVALID_DATA			= -30;		/*データが不正*/
		public const int MLP_NO_LICENSE_KEY			= -31;  	/*ライセンスキーが指定されていません*/
		public const int MLP_UNUSABLE_LICENSE		= -32;		/*このバージョンでは使えないライセンスです*/
		public const int MLP_EXPIRED_LICENSE		= -33;		/*失効したライセンスです*/
		public const int MLP_ILLIGUL_OS_LICENSE		= -34;		/*このOSでは使えないライセンスです*/
		public const int MLP_ILLEGAL_OS_LICENSE		= -34;		/*このOSでは使えないライセンスです*/
		public const int MLP_OUT_OF_BOUND			= -35;		/*値が範囲外です*/
		public const int MLP_COLOR_PROFILE_NOT_FOUND= -36;		/*カラープロファイルを読み込めません*/
		public const int MLP_INVALID_VER_COLOR_PROFILE=-37;		/*ver.2ではないカラープロファイル*/
		public const int MLP_INVALID_COLOR_PROFILE	= -38;		/*不正なカラープロファイル*/
		public const int MLP_FAIL_TO_COUNT_PAGES	= -39;		/*ページ数の取得に失敗しました*/
		public const int MLP_INVALID_PDF_VERSION	= -40;		/*不正なPDF文書のバージョンです*/
		public const int MLP_FONT_NOT_FOUND			= -41;		/*フォントが見つかりません*/
		public const int MLP_ALT_FONT_NOT_FOUND		= -42;		/*代替フォントが見つかりません*/
		public const int MLP_FONT_FILE_NOT_OPENED	= -43;		/*フォントファイルを開けません*/
		public const int MLP_FONT_NOT_LOADED		= -44;		/*フォントをロードできません*/
		public const int MLP_GLYPH_NOT_FOUND		= -51;		/*グリフを検索できません*/
	}

	[StructLayout(LayoutKind.Sequential)] public class OcgData
	{
		[MarshalAs(UnmanagedType.I4)] public int	id;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 101)] public string	name;
		[MarshalAs(UnmanagedType.I4)] public int	level;
		[MarshalAs(UnmanagedType.I4)] public int	visible;
	}

	[StructLayout(LayoutKind.Sequential)] public class OcgUsage
	{
		[MarshalAs(UnmanagedType.I4)] public int	id;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 101)] public string	name;
		[MarshalAs(UnmanagedType.I4)] public int	level;
		[MarshalAs(UnmanagedType.I4)] public int	visible;
		[MarshalAs(UnmanagedType.I4)] public int	printState;
		[MarshalAs(UnmanagedType.I4)] public int	viewState;
	}

	[StructLayout(LayoutKind.Sequential)] public class StrokePathAttr
    {
		[MarshalAs(UnmanagedType.R4)] public float	lineWidth;
		[MarshalAs(UnmanagedType.I4)] public int	lineCap;
		[MarshalAs(UnmanagedType.I4)] public int	lineJoin;
		[MarshalAs(UnmanagedType.R4)] public float	miterLimit;

		[MarshalAs(UnmanagedType.I4)] public int	dashLen;
		[MarshalAs(UnmanagedType.R4)] public float	dashPhase;
		[MarshalAs(UnmanagedType.ByValArray, SizeConst=32)] public float[]	dashData;

		[MarshalAs(UnmanagedType.ByValArray, SizeConst=5)] public float[]	colorfv;
		[MarshalAs(UnmanagedType.LPStr)] public string	pathString;
		[MarshalAs(UnmanagedType.I4)] public int		x, y, width, height;
		[MarshalAs(UnmanagedType.I4)] public int		colors;
	}

	[StructLayout(LayoutKind.Sequential)] public class ImageMatrix
	{
		public float	a, b, c, d, e, f;
	}

	[StructLayout(LayoutKind.Sequential)] public class ImageTileAttr
	{
		[MarshalAs(UnmanagedType.I4)] public int	width, height, colors;
		[MarshalAs(UnmanagedType.R4)] public float	alpha;
		[MarshalAs(UnmanagedType.R4)] public float	xstep, ystep;
									  public ImageMatrix   ctm;
	}

	[StructLayout(LayoutKind.Sequential)] public class PrsFontName
	{
		[MarshalAs(UnmanagedType.I4)] public int	pageNum;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]   public string fontId;
		[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]   public string fontName;
		[MarshalAs(UnmanagedType.I4)] public int	embedded, render;
	}

	public class PdfImager
	{
		private GCHandle gch_rending;
		private GCHandle gch_rended;
		private GCHandle gch_draw_stroke_path;
		private GCHandle gch_define_image_tile;
		private GCHandle gch_define_mask_image_tile;
		private GCHandle gch_draw_image_tiling;
		private bool gch_active;

		public virtual void Rendering(int PageNumber){}
		public virtual void Rendered(int PageNumber){}
		public virtual int DrawStrokePath(StrokePathAttr pa, ref byte[] pixels) { return 1; }
		public virtual void DefineImageTile(ImageTileAttr pa, byte[] pixels) {}
		public virtual void DefineMaskImageTile(ImageTileAttr pa, byte[] pixels) {}
		public virtual int DrawTiledImage(ImageTileAttr ia1, ref byte[] pixels1, ImageTileAttr ia2, ref byte[] pixels2) { return 1; }

		[DllImport("PdfImagerLP.dll")] static extern int MlpInitializeA(string lic);
		[DllImport("PdfImagerLP.dll")] static extern void MlpUninitialize();
		[DllImport("PdfImagerLP.dll")] static extern int MlpLoadInitFileA(string fileName);
		[DllImport("PdfImagerLP.dll")] static extern int MlpLoadInitDataA(string xmlData, int dataLen);
		[DllImport("PdfImagerLP.dll")] static extern int MlpOpenDocA(string fileName, string ownPwd, string usrPwd);
		[DllImport("PdfImagerLP.dll")] static extern int MlpOpenMemDoc(byte[] data, uint len, string ownPwd, string usrPwd);
		[DllImport("PdfImagerLP.dll")] static extern int MlpOpenDocUsageA(string fileName, string pswd, int usage);
		[DllImport("PdfImagerLP.dll")] static extern int MlpOpenMemDocUsage(byte[] data, uint len, string pswd, int usage);
		[DllImport("PdfImagerLP.dll")] static extern void MlpCloseDoc();
		[DllImport("PdfImagerLP.dll")] static extern int MlpValidateOwnerPasswordA(string pass);
		[DllImport("PdfImagerLP.dll")] static extern int MlpValidateUserPasswordA(string pass);
		[DllImport("PdfImagerLP.dll")] static extern int MlpPageCount();
		[DllImport("PdfImagerLP.dll")] static extern int MlpPageToPictA(int pageNum, int dpi, int quality, string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePictA(int pageNum, string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePictPageArrayA(int[] pageNumArray, int length, string outputFileName);
		[DllImport("PdfImagerLP.dll")] static extern int MlpRangeToTiffA(int startPageNum, int endPageNum, int dpi, string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreateTiffRangeA(int startPageNum, int endPageNum, string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern int MlpConvertToTiffA(string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern int MlpMultiPageToTiffA(string pageList, int dpi, string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreateTiffMultiA(string pageList, string outFilePath);
		[DllImport("PdfImagerLP.dll")] static extern void MlpShowLicenseInfo();
		[DllImport("PdfImagerLP.dll")] static extern int MlpLicenseInfoStrA(StringBuilder licenseStr);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetDocumentInfo(int opt, out int flag);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetDocumentInfo(int opt, out string d);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetLibraryInfo(int opt, out int data);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPicturePixel(int pageNum, out int x, out int y);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPicturePixelX(int pageNum);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPicturePixelY(int pageNum);

		[DllImport("PdfImagerLP.dll")] static extern int MlpSetCanvasSize(int x, int y);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetCanvasOrigin(int org);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureSize(int x, int y, int unit);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPicture(int cmd);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPicture(int cmd, int val);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPicture(int cmd, int val1, int val2);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPicture(int cmd, double val);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureRGB();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureCMYK();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureGray();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureDither();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureBW();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureFaxG3();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureFaxG4();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureBufSize(int mb);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureResolution(int dpi);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPictureQuality(int quality);

		[DllImport("PdfImagerLP.dll")] static extern int MlpSetCallbackFunc(int mode, int opt, MulticastDelegate fnc);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetTiffCompress(int mode);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPicturePixelX(int x);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetPicturePixelY(int y);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetEnableAntialis();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetDisableAntialis();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetFillPathAAEnable();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetFillPathAADisable();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetCid0GlyphMode(int mode);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetCid0GlyphMode();
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPageRotation(int pageNum, out int rotate);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPageBoundary(int pageNum, int kind, out float userunit, out float left, out float bottom, out float right, out float top);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPageRawBoundary(int pageNum, int kind, out float userunit, out float left, out float bottom, out float right, out float top);

		[DllImport("PdfImagerLP.dll")] static extern int MlpGetOcgCount();
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetOcgData(out IntPtr ptr);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetOcgUsage(out IntPtr ptr);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetOcgState(IntPtr ptr);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetOcgStateU(IntPtr ptr);

		[DllImport("PdfImagerLP.dll")] static extern int MlpPrintRestriction();

		[DllImport("PdfImagerLP.dll")] static extern int MlpSetRGBProfileA(string fileName);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetCMYKProfileA(string fileName);
		[DllImport("PdfImagerLP.dll")] static extern int MlpUnuseColorProfile();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetColorConversion(int color_convert_direction, int flag);

		[DllImport("PdfImagerLP.dll")] static extern int MlpSetDefaultFontA(string defaultFont);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetAlternateFontA(string org, string alt, int always);
        [DllImport("PdfImagerLP.dll")] static extern int MlpSetAlternateFont2A(string org, string alt, string weight, string style, int always, int replace);
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetAlternateFontExA(string org, string alt, int always, int replace, int replace_on_render2, int alt_bold, int alt_italic, int weight, int style, int variant, IntPtr p);

		[DllImport("PdfImagerLP.dll")] static extern int MlpSetImageWatermarkA(string fileName, int x_position, int y_position, int z_position, float x_scale, float y_scale, float alpha);
		[DllImport("PdfImagerLP.dll", CharSet=CharSet.Unicode)] static extern int MlpSetFillTextWatermarkW(string text, string fontName, int x_position, int y_position, int z_position, float fontSize, string color, float alpha);
		[DllImport("PdfImagerLP.dll", CharSet=CharSet.Unicode)] static extern int MlpSetStrokeTextWatermarkW(string text, string fontName, int x_position, int y_position, int z_position, float fontSize, string color, float alpha);
		[DllImport("PdfImagerLP.dll")] static extern int MlpClearWatermarkData();

		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePictBufPage(int pageNumber, out IntPtr pixels, out int width, out int height, out int n_of_colors);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePictBufPageRange(int firstPageNumber, int lastPageNumber);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePictBufPageArray(int[] pageArray, int length);
		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePictMem(int pageNumber);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPictFromMem(out IntPtr pixels, out int width, out int height, out int n_of_colors);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPictFromMemIndex(int index, out IntPtr pixels, out int width, out int height, out int n_of_colors, out int pageNumber);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPictFromMemIndex(int index, out IntPtr pixels, out int width, out int height, out int n_of_colors, IntPtr pageNumber);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetPictFromMemPage(int pageNum, out IntPtr pixels, out int width, out int height, out int n_of_colors);
		[DllImport("PdfImagerLP.dll")] static extern int MlpFreePictMem();

		[DllImport("PdfImagerLP.dll")] static extern int MlpCreatePanel(int width, int height, int colorspace);
		[DllImport("PdfImagerLP.dll")] static extern int MlpClearPanel();
		[DllImport("PdfImagerLP.dll")] static extern int MlpFreePanel();
		[DllImport("PdfImagerLP.dll")] static extern int MlpPanelToFileA(string fileName);
		[DllImport("PdfImagerLP.dll")] static extern int MlpMemPictToPanel(int xpos, int ypos, float alpha);
		[DllImport("PdfImagerLP.dll")] static extern int MlpMemPictToPanelBBox(int panelXPos, int panelYPos, float alpha, int bboxXPos, int bboxYPos, int bboxWidth, int bboxHeight);
		[DllImport("PdfImagerLP.dll")] static extern int MlpPaintRectToPanel(int xpos, int ypos, int width, int height, int lineWidth, byte[] color);

		[DllImport("PdfImagerLP.dll")] static extern int MlpSetTextWidgetMultilineFontSize(float point);
		[DllImport("PdfImagerLP.dll")] static extern float MlpGetTextWidgetMultilineFontSize();
		[DllImport("PdfImagerLP.dll")] static extern int MlpSetTextWidgetMultilineUseLastSize(int flag);
		[DllImport("PdfImagerLP.dll")] static extern int MlpGetTextWidgetMultilineUseLastSize();

		[DllImport("PdfImagerLP.dll")] static extern int MlpPrsGetFontInfo(out IntPtr data, out int len);
		[DllImport("PdfImagerLP.dll")] static extern int MlpPrsGetFontInfoWithTr(out IntPtr data, out int len);

		public PdfImager()
		{
			gch_active = false;
		}

		~PdfImager()
		{
			if (gch_active)
			{
				gch_rending.Free();
				gch_rended.Free();
				gch_draw_stroke_path.Free();
				gch_define_image_tile.Free();
				gch_define_mask_image_tile.Free();
				gch_draw_image_tiling.Free();
			}
		}

		public int DrawStrokePath_P(IntPtr pa, IntPtr pixels)
		{
			StrokePathAttr spa = (StrokePathAttr)Marshal.PtrToStructure(pa, typeof(StrokePathAttr));
			int len = spa.width * spa.height * spa.colors;
			byte[] pix = new byte[len];
			Marshal.Copy(pixels, pix, 0, len);
			int res = DrawStrokePath(spa, ref pix);
			if(res == 1) Marshal.Copy(pix, 0, pixels, len);

			return res; 
		}

		public void DefineImageTile_P(IntPtr ia, IntPtr pixels)
		{
			ImageTileAttr sia = (ImageTileAttr)Marshal.PtrToStructure(ia, typeof(ImageTileAttr));
			int len = sia.width * sia.height * sia.colors;
			byte[] pix = new byte[len];
			if(pixels != IntPtr.Zero) Marshal.Copy(pixels, pix, 0, len);
			DefineImageTile(sia, pix);
		}

		public void DefineMaskImageTile_P(IntPtr ia, IntPtr pixels)
		{
			ImageTileAttr sia = (ImageTileAttr)Marshal.PtrToStructure(ia, typeof(ImageTileAttr));
			int len = sia.width * sia.height * sia.colors;
			byte[] pix = new byte[len];
			if(pixels != IntPtr.Zero) Marshal.Copy(pixels, pix, 0, len);
			DefineMaskImageTile(sia, pix);
		}

		public int DrawTiledImage_P(IntPtr ia1, IntPtr pixels1, IntPtr ia2, IntPtr pixels2)
		{
			ImageTileAttr sia1 = (ImageTileAttr)Marshal.PtrToStructure(ia1, typeof(ImageTileAttr));
			ImageTileAttr sia2 = (ImageTileAttr)Marshal.PtrToStructure(ia2, typeof(ImageTileAttr));
			int len1 = sia1.width * sia1.height * sia1.colors;
			int len2 = sia2.width * sia2.height * sia2.colors;
			byte[] pix1 = new byte[len1];
			byte[] pix2 = new byte[len2];
			if(pixels1 != IntPtr.Zero) Marshal.Copy(pixels1, pix1, 0, len1);
			if(pixels2 != IntPtr.Zero) Marshal.Copy(pixels2, pix2, 0, len2);
			int res = DrawTiledImage(sia1, ref pix1, sia2, ref pix2);
			if (res == 1)
			{
				if(pixels1 != IntPtr.Zero) Marshal.Copy(pix1, 0, pixels1, len1);
				if(pixels2 != IntPtr.Zero) Marshal.Copy(pix2, 0, pixels2, len2);
			}

			return res;
		}

		public int Initialize(string license)
		{
			int ret = MlpInitializeA(license);
			if (ret == 0)
			{
				DelegateRendering rending = new DelegateRendering(Rendering);
				DelegateRendered rended = new DelegateRendered(Rendered);
				DelegateDrawStrokePath drawStrokePath = new DelegateDrawStrokePath(DrawStrokePath_P);
				DelegateDefineImageTile defineImageTile = new DelegateDefineImageTile(DefineImageTile_P);
				DelegateDefineMaskImageTile defineMaskImageTile = new DelegateDefineMaskImageTile(DefineMaskImageTile_P);
				DelegateDrawTiledImage drawTiledImage = new DelegateDrawTiledImage(DrawTiledImage_P);
				gch_rending = GCHandle.Alloc(rending);
				gch_rended = GCHandle.Alloc(rended);
				gch_draw_stroke_path = GCHandle.Alloc(drawStrokePath);
				gch_define_image_tile = GCHandle.Alloc(defineImageTile);
				gch_define_mask_image_tile = GCHandle.Alloc(defineMaskImageTile);
				gch_draw_image_tiling = GCHandle.Alloc(drawTiledImage);
				gch_active = true;
				SetCallbackFunc(CallbackMode.MLP_RENDERING_CB, 0, rending);
				SetCallbackFunc(CallbackMode.MLP_RENDERED_CB, 0, rended);
				SetCallbackFunc(CallbackMode.MLP_DRAW_STROKE_PATH_CB, 0, drawStrokePath);
				SetCallbackFunc(CallbackMode.MLP_DEFINE_IMAGE_TILE_CB, 0, defineImageTile);
				SetCallbackFunc(CallbackMode.MLP_DEFINE_MASK_IMAGE_TILE_CB, 0, defineMaskImageTile);
				SetCallbackFunc(CallbackMode.MLP_DRAW_TILED_IMAGE_CB, 0, drawTiledImage);
				SetPicture(PictureOpt.STROKEPATH_ENABLE_CALLBACK, 0);	/*disable callback*/
				SetPicture(PictureOpt.TILE_IMAGE_ENABLE_CALLBACK, 0);	/*disable callback*/
				SetPicture(PictureOpt.TILE_MASK_IMG_ENABLE_CALLBACK, 0);/*disable callback*/
				SetPicture(PictureOpt.TILING_ENABLE_CALLBACK, 0);		/*disable callback*/
			}
			return ret;
		}

		public int LoadInitFile(string initFileName)
		{
			return MlpLoadInitFileA(initFileName);
		}

		public int LoadInitData(string xmlData, int dataLen)
		{
			return MlpLoadInitDataA(xmlData, dataLen);
		}
		public int LoadInitData(string xmlData)
		{
			return MlpLoadInitDataA(xmlData, xmlData.Length + 1);
		}

		public int OpenDoc(string fileName, string ownerPassword, string userPassword)
		{
			return MlpOpenDocA(fileName, ownerPassword, userPassword);
		}

        public int OpenMemDoc(byte[] data, uint len, string ownerPassword, string userPassword)
        {
            return MlpOpenMemDoc(data, len, ownerPassword, userPassword);
        }

		public int OpenDocUsage(string fileName, string password, int usage)
		{
			return MlpOpenDocUsageA(fileName, password, usage);
		}

		public int OpenMemDocUsage(byte[] data, uint len, string password, int usage)
		{
			return MlpOpenMemDocUsage(data, len, password, usage);
		}

		public void CloseDoc()
		{
			MlpCloseDoc();
		}

		public bool AuthenticateOwnerPassword(string password)
		{
			return MlpValidateOwnerPasswordA(password)==0 ? false : true;
		}

		public bool AuthenticateUserPassword(string password)
		{
			return MlpValidateUserPasswordA(password)==0 ? false : true;
		}

		public void Uninitialize()
		{
			MlpUninitialize();
		}

		public int PageCount()
		{
			return MlpPageCount();
		}

		public int GetOcgCount()
		{
			return MlpGetOcgCount();
		}

		public OcgData[] GetOcgData()
		{
			int c = MlpGetOcgCount();
			if (c == 0)
			{
				return null;
			}
			OcgData[] data = new OcgData[c];
			IntPtr ptr = IntPtr.Zero;
			if (MlpGetOcgData(out ptr) != 0)
			{
				return null;
			}
			IntPtr pos = new IntPtr(ptr.ToInt64());
			for (int i = 0; i < c; i++)
			{
				data[i] = (OcgData)Marshal.PtrToStructure(pos, typeof(OcgData));
				pos = (IntPtr)((int)pos + Marshal.SizeOf(data[i]));
			}

			return data;
		}

		public OcgUsage[] GetOcgUsage()
		{
			int c = MlpGetOcgCount();
			if (c == 0)
			{
				return null;
			}
			OcgUsage[] data = new OcgUsage[c];
			IntPtr ptr = IntPtr.Zero;
			if (MlpGetOcgUsage(out ptr) != 0)
			{
				return null;
			}
			IntPtr pos = new IntPtr(ptr.ToInt64());
			for (int i = 0; i < c; i++)
			{
				data[i] = (OcgUsage)Marshal.PtrToStructure(pos, typeof(OcgUsage));
				pos = (IntPtr)((int)pos + Marshal.SizeOf(data[i]));
			}

			return data;
		}

		public int SetOcgState(OcgData[] data)
		{
			int len = data.Length;
			IntPtr ptr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(OcgData)) * len);
			IntPtr pos = new IntPtr(ptr.ToInt64());
			for (int i = 0; i < len; i++)
			{
				Marshal.StructureToPtr(data[i], pos, true);
				pos = (IntPtr)((int)pos + Marshal.SizeOf(data[i]));
			}
			int res = MlpSetOcgState(ptr);
			Marshal.FreeHGlobal(ptr);

			return res;
		}

		public int SetOcgStateU(OcgUsage[] data)
		{
			int len = data.Length;
			IntPtr ptr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(OcgUsage)) * len);
			IntPtr pos = new IntPtr(ptr.ToInt64());
			for (int i = 0; i < len; i++)
			{
				Marshal.StructureToPtr(data[i], pos, true);
				pos = (IntPtr)((int)pos + Marshal.SizeOf(data[i]));
			}
			int res = MlpSetOcgStateU(ptr);
			Marshal.FreeHGlobal(ptr);

			return res;
		}

		public int PrintRestriction()
		{
			return MlpPrintRestriction();
		}

		public int SetRGBProfile(string fileName)
		{
			return MlpSetRGBProfileA(fileName);
		}

		public int SetCMYKProfile(string fileName)
		{
			return MlpSetCMYKProfileA(fileName);
		}

		public int UnuseColorProfile()
		{
			return MlpUnuseColorProfile();
		}

		public int SetColorConversion(int color_convert_direction, int flag)
		{
			return MlpSetColorConversion(color_convert_direction, flag);
		}
		public int SetColorConvertForImage(int color_convert_direction, int flag)	//no longer support
		{
			return MlpSetColorConversion(color_convert_direction, flag);
		}

		public int SetDefaultFont(string defaultFontName)
		{
			return MlpSetDefaultFontA(defaultFontName);
		}

		public int SetAlternateFont(string originalFontName, string alternateFontName, int always)
		{
			return MlpSetAlternateFontA(originalFontName, alternateFontName, always);
		}

		public int SetAlternateFont2(string originalFontName, string alternateFontName, string fontWeight, string fontStyle, int always, int replace)
		{
			return MlpSetAlternateFont2A(originalFontName, alternateFontName, fontWeight, fontStyle, always, replace);
		}

		public int SetAlternateFontEX(string originalFontName, string alternateFontName, int always, int replace, int replace_on_render2, int alt_bold, int alt_italic, int weight, int style, int variant)
		{
			return MlpSetAlternateFontExA(originalFontName, alternateFontName, always, replace, replace_on_render2, alt_bold, alt_italic, weight, style, 0, IntPtr.Zero);
		}

		public int SetAlternateFontEX(string originalFontName, string alternateFontName, int always, int replace, int replace_on_render2, int alt_bold, int alt_italic, int weight, int style, int variant, ImageMatrix transform)
		{
			IntPtr ptr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(ImageMatrix)));
			Marshal.StructureToPtr(transform, ptr, true);
			return MlpSetAlternateFontExA(originalFontName, alternateFontName, always, replace, replace_on_render2, alt_bold, alt_italic, weight, style, 0, ptr);
		}

		public int SetAlternateFontEX(string originalFontName, string alternateFontName, bool always, bool replace, bool replace_on_render2, int alt_bold, bool alt_italic, int weight, int style, int variant)
		{
			return MlpSetAlternateFontExA(originalFontName, alternateFontName, always ? 1 : 0, replace ? 1 : 0, replace_on_render2 ? 1 : 0, alt_bold, alt_italic ? 1 : 0, weight, style, 0, IntPtr.Zero);
		}

		public int SetAlternateFontEX(string originalFontName, string alternateFontName, bool always, bool replace, bool replace_on_render2, int alt_bold, bool alt_italic, int weight, int style, int variant, ImageMatrix transform)
		{
			IntPtr ptr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(ImageMatrix)));
			Marshal.StructureToPtr(transform, ptr, true);
			return MlpSetAlternateFontExA(originalFontName, alternateFontName, always ? 1 : 0, replace ? 1 : 0, replace_on_render2 ? 1 : 0, alt_bold, alt_italic ? 1 : 0, weight, style, 0, ptr);
		}

		public int SetImageWatermark(string fileName, int x_position, int y_position, int z_position, float x_scale, float y_scale, float alpha)
		{
			return MlpSetImageWatermarkA(fileName, x_position, y_position, z_position, x_scale, y_scale, alpha);
		}

		public int SetFillTextWatermark(string text, string fontName, int x_position, int y_position, int z_position, float fontSize, string color, float alpha)
		{
			return MlpSetFillTextWatermarkW(text, fontName, x_position, y_position, z_position, fontSize, color, alpha);
		}

		public int SetStrokeTextWatermark(string text, string fontName, int x_position, int y_position, int z_position, float fontSize, string color, float alpha)
		{
			return MlpSetStrokeTextWatermarkW(text, fontName, x_position, y_position, z_position, fontSize, color, alpha);
		}

		public int ClearWatermarkData()
		{
			return MlpClearWatermarkData();
		}

		public int PageToPict(int pageNumber, int dpi, int quality, string outputFilePath)
		{
			return MlpPageToPictA(pageNumber, dpi, quality, outputFilePath);
		}

		public int CreatePict(int pageNumber, string outputFilePath)
		{
			return MlpCreatePictA(pageNumber, outputFilePath);
		}

		public int CreatePictBufPage(int pageNumber, out byte[] pixels, out int width, out int height, out int nOfColors)
		{
			pixels = null;
			width = height = nOfColors = 0;

			int ret = MlpCreatePictMem(pageNumber);
			if (ret != 0) return ret;
			ret = GetPictFromMem(out pixels, out width, out height, out nOfColors);

			return ret;
		}

		public int CreatePictBufPageRange(int firstPageNumber, int lastPageNumber)
		{
			return MlpCreatePictBufPageRange(firstPageNumber, lastPageNumber);
		}

		public int CreatePictBufPageArray(int[] pageArray)
		{
			return MlpCreatePictBufPageArray(pageArray, pageArray.Length);
		}

		public int CreatePictBufPageArray(int[] pageArray, int length)
		{
			if (pageArray.Length < length) return MlpCreatePictBufPageArray(pageArray, pageArray.Length);
			else return MlpCreatePictBufPageArray(pageArray, length);
		}

		public int CreatePictMem(int pageNumber)
		{
			return MlpCreatePictMem(pageNumber);
		}

		public int GetPictFromMem(out byte[] pixels, out int width, out int height, out int nOfColors)
		{
			IntPtr data;
			int ret = 0;

			pixels = null;
			width = height = nOfColors = 0;
			ret = MlpGetPictFromMem(out data, out width, out height, out nOfColors);
			if (ret != 0) return ret;
			if (data == IntPtr.Zero) return err.MLP_MEMORY_ERROR;
			int len = width * height * nOfColors;
			try
			{
				pixels = new byte[len];
			}
			catch
			{
				return err.MLP_MEMORY_ERROR;
			}
			Marshal.Copy(data, pixels, 0, len);

			return ret;
		}

		public int GetPictFromMemIndex(int index, out byte[] pixels, out int width, out int height, out int nOfColors, out int pageNumber)
		{
			IntPtr data;
			int ret = 0;

			pixels = null;
			width = height = nOfColors = 0;
			ret = MlpGetPictFromMemIndex(index, out data, out width, out height, out nOfColors, out pageNumber);
			if (ret != 0) return ret;
			if (data == IntPtr.Zero) return err.MLP_MEMORY_ERROR;
			int len = width * height * nOfColors;
			try
			{
				pixels = new byte[len];
			}
			catch
			{
				return err.MLP_MEMORY_ERROR;
			}
			Marshal.Copy(data, pixels, 0, len);
			return ret;
		}

		public int GetPictFromMemIndex(int index, out byte[] pixels, out int width, out int height, out int nOfColors)
		{
			IntPtr data;
			int ret = 0;

			pixels = null;
			width = height = nOfColors = 0;
			ret = MlpGetPictFromMemIndex(index, out data, out width, out height, out nOfColors, IntPtr.Zero);
			if (ret != 0) return ret;
			if (data == IntPtr.Zero) return err.MLP_MEMORY_ERROR;
			int len = width * height * nOfColors;
			try
			{
				pixels = new byte[len];
			}
			catch
			{
				return err.MLP_MEMORY_ERROR;
			}
			Marshal.Copy(data, pixels, 0, len);
			return ret;
		}

		public int GetPictFromMemPage(int pageNumber, out byte[] pixels, out int width, out int height, out int nOfColors)
		{
			IntPtr data;
			int ret = 0;

			pixels = null;
			width = height = nOfColors = 0;
			ret = MlpGetPictFromMemPage(pageNumber, out data, out width, out height, out nOfColors);
			if (ret != 0) return ret;
			if (data == IntPtr.Zero) return err.MLP_MEMORY_ERROR;
			int len = width * height * nOfColors;
			try
			{
				pixels = new byte[len];
			}
			catch
			{
				return err.MLP_MEMORY_ERROR;
			}
			Marshal.Copy(data, pixels, 0, len);
			return ret;
		}

		public int GetFreePictMem()
		{
			return MlpFreePictMem();
		}

		public int ConvertToTiff(string outputFilePath)
		{
			return MlpConvertToTiffA(outputFilePath);
		}

		public int RangeToTiff(int startPageNumber, int endPageNumber, int dpi, string outputFilePath)
		{
			return MlpRangeToTiffA(startPageNumber, endPageNumber, dpi, outputFilePath);
		}

		public int CreateTiffRange(int startPageNumber, int endPageNumber, string outputFilePath)
		{
			return MlpCreateTiffRangeA(startPageNumber, endPageNumber, outputFilePath);
		}

		public int MultiPageToTiff(string pageList, int dpi, string outputFilePath)
		{
			return MlpMultiPageToTiffA(pageList, dpi, outputFilePath);
		}

		public int CreateTiffMulti(string pageList, string outputFilePath)
		{
			return MlpCreateTiffMultiA(pageList, outputFilePath);
		}

		public void ShowLicenseInfo()
		{
			MlpShowLicenseInfo();
		}

		public int GetLicenseInfoStr(StringBuilder licStr)
		{
			return MlpLicenseInfoStrA(licStr);
		}

		public int GetDocumentInfo(DocumentOpt opt, out int flag)
		{
			return MlpGetDocumentInfo((int)opt, out flag);
		}

		public int GetDocumentInfo(DocumentOpt opt, out string d)
		{
			return MlpGetDocumentInfo((int)opt, out d);
		}

		public int GetLibraryInfo(LibraryOpt opt, out int data)
		{
			return MlpGetLibraryInfo((int)opt, out data);
		}

		public int GetPicturePixel(int pageNum, out int x, out int y)
		{
			return MlpGetPicturePixel(pageNum, out x, out y);
		}

		public int GetPicturePixelX(int pageNum)
		{
			return MlpGetPicturePixelX(pageNum);
		}

		public int GetPicturePixelY(int pageNum)
		{
			return MlpGetPicturePixelY(pageNum);
		}

		public int SetCanvasSize(int x, int y)
		{
			return MlpSetCanvasSize(x, y);
		}

		public int SetCanvasOrigin(int org)
		{
			return MlpSetCanvasOrigin(org);
		}

		public int SetCanvasOrigin(CanvasOrigin org)
		{
			return MlpSetCanvasOrigin((int)org);
		}

		public int SetPictureSize(int x, int y, PictureUnit unit)
		{
			return MlpSetPictureSize(x, y, (int)unit);
		}

		public int SetPicture(PictureOpt opt)
		{
			return MlpSetPicture((int)opt);
		}

		public int SetPicture(PictureOpt opt, int val)
		{
			return MlpSetPicture((int)opt, val);
		}

		public int SetPicture(PictureOpt opt, int val1, int val2)
		{
			if (opt == PictureOpt.XY_PIXEL_SIZE_MEM)
				return MlpSetPicture((int)opt, val1, val2);
			else
				return err.MLP_INVALID_CMD;
		}

		public int SetPicture(PictureOpt opt, float val)
		{
			return MlpSetPicture((int)opt, (double)val);
		}

		public int SetPicture(PictureOpt opt, double val)
		{
			return MlpSetPicture((int)opt, val);
		}

		public int SetPicture(PictureOpt opt, bool b)
		{
			return MlpSetPicture((int)opt, b ? (int)1 : (int) 0);
		}

		public int SetPictureRGB()
		{
			return MlpSetPictureRGB();
		}

		public int SetPictureCMYK()
		{
			return MlpSetPictureCMYK();
		}

		public int SetPictureGray()
		{
			return MlpSetPictureGray();
		}

		public int SetPictureDither()
		{
			return MlpSetPictureDither();
		}

		public int SetPictureBW()
		{
			return MlpSetPictureBW();
		}

		public int SetPictureFaxG3()
		{
			return MlpSetPictureFaxG3();
		}

		public int SetPictureFaxG4()
		{
			return MlpSetPictureFaxG4();
		}

		public int SetPictureBufSize(int mb)
		{
			return MlpSetPictureBufSize(mb);
		}

		public int SetPictureResolution(int dpi)
		{
			return MlpSetPictureResolution(dpi);
		}

		public int SetPictureQuality(int quality)
		{
			return MlpSetPictureQuality(quality);
		}

        public int SetCallbackFunc(CallbackMode mode, int opt, MulticastDelegate fnc)
		{
			return MlpSetCallbackFunc((int)mode, opt, fnc);
		}

		public int SetTiffCompress(TiffCompressMode mode)
		{
			return MlpSetTiffCompress((int)mode);
		}

		public int SetPicturePixelX(int x)
		{
			return MlpSetPicturePixelX(x);
		}

		public int SetPicturePixelY(int y)
		{
			return MlpSetPicturePixelY(y);
		}

		public int SetEnableAntialis()
		{
			return MlpSetEnableAntialis();
		}

		public int SetDisableAntialis()
		{
			return MlpSetDisableAntialis();
		}

		public int SetFillPathAAEnable()
		{
			return MlpSetFillPathAAEnable();
		}

		public int SetFillPathAADisable()
		{
			return MlpSetFillPathAADisable();
		}

		public int SetCid0GlypgMode(Cid0GlyphMode mode)
		{
			return MlpSetCid0GlyphMode((int)mode);
		}

		public Cid0GlyphMode GetCid0GlypgMode()
		{
			return (Cid0GlyphMode)MlpGetCid0GlyphMode();
		}

		public Cid0GlyphMode Cid0GlypgMode
		{
			set
			{
				MlpSetCid0GlyphMode((int)value);
			}
			get
			{
				return (Cid0GlyphMode)MlpGetCid0GlyphMode();
			}
		}

		public int GetPageRotation(int pageNum, out int rotate)
		{
			return MlpGetPageRotation(pageNum, out rotate);
		}

		public int GetPageBoundary(int pageNum, BoundaryBox kind, out float userUnit, out float left, out float bottom, out float right, out float top)
		{
			return MlpGetPageBoundary(pageNum, (int)kind, out userUnit, out left, out bottom, out right, out top);
		}

		public int GetPageRawBoundary(int pageNum, BoundaryBox kind, out float userUnit, out float left, out float bottom, out float right, out float top)
		{
			return MlpGetPageRawBoundary(pageNum, (int)kind, out userUnit, out left, out bottom, out right, out top);
		}

		public int CreatePanel(int width, int height, PictureOpt opt)
		{
			return MlpCreatePanel(width, height, (int)opt);
		}

		public int ClearPanel()
		{
			return MlpClearPanel();
		}

		public int FreePanel()
		{
			return MlpFreePanel();
		}

		public int PanelToFile(string fileName)
		{
			return MlpPanelToFileA(fileName);
		}

		public int MemPictToPanel(int xPos, int yPos, float alpha)
		{
			return MlpMemPictToPanel(xPos, yPos, alpha);
		}

		public int MemPictToPanelBBox(int xPos, int yPos, float alpha, int bboxXPos, int bboxYPos, int bboxWidth, int bboxHeight)
		{
			return MlpMemPictToPanelBBox(xPos, yPos, alpha, bboxXPos, bboxYPos, bboxWidth, bboxHeight);
		}

		public int PaintRectToPanel(int xPos, int yPos, int width, int height, int lineWidth, byte[] color)
		{
			byte[] defaultColor = new byte[5];
			if(color != null) for (int i = 0; i < color.Length; i++) defaultColor[i] = color[i];
			return MlpPaintRectToPanel(xPos, yPos, width, height, lineWidth, defaultColor);
		}

		public int LogicalCPUCount
		{
			get
			{
				int c;
				GetLibraryInfo(LibraryOpt.LOGICAL_CPU_COUNT, out c);
				return c;
			}
		}

		public bool ThreadSplitPage
		{
			get
			{
				int b;
				GetLibraryInfo(LibraryOpt.THREAD_SPLIT_PAGE, out b);
				if (b == 0) return false;
				return true;
			}
			set
			{
				SetPicture(PictureOpt.THREAD_SPLIT_PAGE, value);
			}
		}

		public int ThreadSplitPageCount
		{
			get
			{
				int c;
				GetLibraryInfo(LibraryOpt.THREAD_SPLIT_PAGE_COUNT, out c);
				return c;
			}
			set
			{
				SetPicture(PictureOpt.THREAD_SPLIT_PAGE_COUNT, (int)value);
			}
		}

		public float TextWidgetMultilineFontSize
		{
			set
			{
				MlpSetTextWidgetMultilineFontSize(value);
			}
			get
			{
				return MlpGetTextWidgetMultilineFontSize();
			}
		}

		public int TextWidgetMultilineUseLastSize
		{
			set
			{
				MlpSetTextWidgetMultilineUseLastSize(value == 0 ? 0 : 1);
			}
			get
			{
				return MlpGetTextWidgetMultilineUseLastSize();
			}
		}

		public int PrsGetFontInfo(out PrsFontName[] fnArry, out int len)
		{
			IntPtr ptr = IntPtr.Zero;
			int ret;

			if ((ret = MlpPrsGetFontInfo(out ptr, out len)) != 0)
			{
				fnArry = null;
				return ret;
			}
			fnArry = new PrsFontName[len];
			IntPtr pos = new IntPtr(ptr.ToInt64());
			for (int i = 0; i < len; i++)
			{
				fnArry[i] = (PrsFontName)Marshal.PtrToStructure(pos, typeof(PrsFontName));
				pos = (IntPtr)((int)pos + Marshal.SizeOf(fnArry[i]));
			}
			return 0;
		}

		public int PrsGetFontInfoWithTr(out PrsFontName[] fnArry, out int len)
		{
			IntPtr ptr = IntPtr.Zero;
			int ret;

			if ((ret = MlpPrsGetFontInfoWithTr(out ptr, out len)) != 0)
			{
				fnArry = null;
				return ret;
			}
			fnArry = new PrsFontName[len];
			IntPtr pos = new IntPtr(ptr.ToInt64());
			for (int i = 0; i < len; i++)
			{
				fnArry[i] = (PrsFontName)Marshal.PtrToStructure(pos, typeof(PrsFontName));
				pos = (IntPtr)((int)pos + Marshal.SizeOf(fnArry[i]));
			}
			return 0;
		}

#if false && DEBUG 
		public int CreatePngFromPixels(byte[] pixels, int width, int height, int nOfColors, string fileName)
		{
			int len = width * height * nOfColors;
			IntPtr data;
			int ret = 0;

			if ((pixels == null) || (pixels.Length != len) || (fileName.Length < 1)) return err.MLP_INVALID_ARG_VALUE;
			try
			{
				data = Marshal.AllocHGlobal(len);
			}
			catch
			{
				return err.MLP_MEMORY_ERROR;
			}

			try
			{
				Marshal.Copy(pixels, 0, data, len);
				ret = MlpCreatePngFromPixels(data, width, height, nOfColors, fileName);
			}
			finally
			{
				Marshal.FreeHGlobal(data);
			}

			return ret;
		}

		public int CreatePngFromPixelsIndex(int index, string fn)
		{
			return MlpCreatePngFromPixelsIndex(index, fn);
		}

		public int CreatePngFromPixelsPageNum(int pageNumber, string fn)
		{
			return MlpCreatePngFromPixelsPageNum(pageNumber, fn);
		}

		public string GetFirstFontName()
		{
			IntPtr firstFontName = MlpGetFirstFontNameW();
			if (firstFontName == IntPtr.Zero)
				return (string)null;
			else
				return Marshal.PtrToStringUni(firstFontName);
		}

		public string GetNextFontName()
		{
			IntPtr nextFontName = MlpGetNextFontNameW();
			if (nextFontName == IntPtr.Zero)
				return (string)null;
			else
				return Marshal.PtrToStringUni(nextFontName);
		}

		public string GetFontType()
		{
			IntPtr fontType = MlpGetFontTypeW();
			if (fontType == IntPtr.Zero)
				return (string)null;
			else
				return Marshal.PtrToStringUni(fontType);
		}
#endif
	}
}
